struct config
{
    char * root_device;
    char * info_partition_device;
    char * data_partition_device;
    char * mapped_device_name;
    char * mapped_device_path;
    char * tmp_data_location;
    char * notify_command;
};
extern struct config config;
struct config config;

void print_usage()
{
    printf("Usage:\n");
    printf("  cryptops-api --root-device=DEV --crypt-name=LABEL "
        " [--notify-command=COMMAND]\n");
}

void set_config_from_arguments(int argc, char ** argv)
{
    // Set defaults.
    char * crypt_name = NULL;
    char * notify_command = "";
    char * root_device = NULL;

    static struct option long_options[] =
    {
        {"crypt-name"    ,  required_argument, 0, 'c'},
        {"notify-command",  required_argument, 0, 'n'},
        {"root-device"   ,  required_argument, 0, 'r'},
        {0, 0, 0, 0}
    };
    int option_index = 0;
    int c;

    while (true)
    {
        c = getopt_long(argc, argv, "c:n:r:", long_options, &option_index);

        // Check if we have dealt with all options.
        if (c == -1)
        {
            break;
        }

        switch (c)
        {
            case 'c':
                crypt_name = optarg;
                break;
            case 'n':
                notify_command = optarg;
                break;
            case 'r':
                root_device = optarg;
                break;
            case '?':
                // getopt_long already printed an error message.
                break;
            default:
                abort();
        }
    }

    // Print any remaining command line arguments (not options).
    if (optind < argc)
    {
        printf("non-option ARGV-elements: ");
        while (optind < argc)
        {
            printf("%s ", argv[optind++]);
        }
        putchar('\n');
    }

    if (root_device == NULL)
    {
        printf("Required option --root-device missing.\n");
        print_usage();
        exit(1);
    }
    if (crypt_name == NULL)
    {
        printf("Required option --crypt-name missing.\n");
        print_usage();
        exit(1);
    }

    config.root_device = root_device;
    config.mapped_device_name = crypt_name;
    config.notify_command = notify_command;

    asprintf(&config.info_partition_device, "%s1", config.root_device);
    asprintf(&config.data_partition_device, "%s2", config.root_device);
    asprintf(&config.mapped_device_path, "/dev/mapper/%s", config.mapped_device_name);
    asprintf(&config.tmp_data_location, "/tmp/%s", config.mapped_device_name);
}
